/*
Copyright (c) 2011, Tom Van Cutsem, Vrije Universiteit Brussel
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the Vrije Universiteit Brussel nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.*/

/**
 * Divide-and-conquer array summation using native Java threads.
 * Primarily meant to show the constant-factor overheads of native threads
 * compared to using a dedicated framework such as Fork/Join.
 * 
 * Based on Dan Grossman's lecture notes
 * "A Sophomoric Introduction to Shared-Memory Parallelism and Concurrency"
 * {@link http://www.cs.washington.edu/homes/djg/teachingMaterials/}
 * 
 * @author tvcutsem
 */
public class DivSumThread extends Thread {
	
	private static final int SEQUENTIAL_CUTOFF = 1000;
	
	private int lo;
	private int hi;
	private int[] arr; //fields to know what to do
	public  int ans; // to return answer
	
	DivSumThread(int[] a, int l, int h) {
		lo = l;
		hi = h;
		arr = a;
	}
	
	public void run() {// set answer
      if ((hi - lo) < SEQUENTIAL_CUTOFF) {
    	  int ans = 0;
    	  for(int i=lo; i < hi; i++)
            ans += arr[i];
    	 this.ans = ans;
      } else {
    	  DivSumThread left = new DivSumThread(arr,lo,(hi+lo)/2);
    	  DivSumThread right= new DivSumThread(arr,(hi+lo)/2,hi);
    	  left.start(); right.start();
    	  try {
    		left.join(); 
			right.join();
		  } catch (InterruptedException e) { e.printStackTrace(); }
          this.ans = left.ans + right.ans;
      }
	}
		
	static int sum(int[] arr) {
	  DivSumThread root = new DivSumThread(arr,0,arr.length);
	  root.start();
	  try {
		root.join();
	  } catch (InterruptedException e) {
		e.printStackTrace();
	  }
	  return root.ans;
	}
	
	public static void main(String[] args) throws Exception {
		int SIZE = 10000 * 4;
		int[] array = new int[SIZE];
		for (int i = 0; i < array.length; i++) {
			array[i] = i;
		}
		long start = System.currentTimeMillis();
		int theSum = sum(array);
		long end = System.currentTimeMillis();
		System.out.println("Sum "+theSum+ " computed in "+(end-start)+" ms");
	}
	
}