/*
Copyright (c) 2011, Tom Van Cutsem, Vrije Universiteit Brussel
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the Vrije Universiteit Brussel nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.*/
/**
 * Naive implementation of summing an array in parallel.
 * 
 * Based on Dan Grossman's lecture notes (lecture 1)
 * "A Sophomoric Introduction to Shared-Memory Parallelism and Concurrency"
 * {@link http://www.cs.washington.edu/homes/djg/teachingMaterials/}
 * 
 * @author tvcutsem
 */
public class SumThread extends java.lang.Thread {
	
	private int lo;
	private int hi;
	private int[] arr; //fields to know what to do
	private int ans = 0; // for communicating result
	
	SumThread(int[] a, int l, int h) {
		arr = a;
		lo = l;
		hi = h;
	}
	
	public void run() {
		for (int i = lo; i < hi; i++) {
			ans += arr[i];
		}
	}
	
	static int sum(int[] arr, int N) throws InterruptedException {
		int len = arr.length;
		int ans = 0;
		SumThread[] ts = new SumThread[N];
		for(int i=0; i < N; i++) { // do parallel computations
			ts[i] = new SumThread(arr,i*len/N,(i+1)*len/N);
		  ts[i].start();
	  }
		for(int i=0; i < N; i++) { // combine results
		  ts[i].join(); // wait for helper to finish!
		  ans += ts[i].ans;
		}
		return ans;
	}
	
	public static void main(String[] args) throws Exception {
		int N = 4;
		int SIZE = 10000 * N;
		int[] array = new int[SIZE];
		for (int i = 0; i < array.length; i++) {
			array[i] = i;
		}
		long start = System.currentTimeMillis();
		int theSum = sum(array, N);
		long end = System.currentTimeMillis();
		System.out.println("Sum "+theSum+ " computed in "+(end-start)+" ms");
	}
}